package de.dpma.register.ws;

import java.net.MalformedURLException;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;

import de.dpma.register.ws.gsm.DPMAregisterGsmService;
import de.dpma.register.ws.marke.DPMAregisterMarkeService;
import de.dpma.register.ws.patent.DPMAregisterPatService;

/**
 * Client für die WebServices des DPMAregister.
 * 
 * @author Michael Fuchs
 */
public class DPMAregisterClient {

    private static final String GSM_SERVICE_WSDL = "/dpmaws/services/DPMAregisterGsmService?wsdl";
    private static final String MARKE_SERVICE_WSDL = "/dpmaws/services/DPMAregisterMarkeService?wsdl";
    private static final String PATENT_SERVICE_WSDL = "/dpmaws/services/DPMAregisterPatService?wsdl";

    private static final QName QNAME_GSM = new QName("http://ws.gsm.register.dpma.de", "DPMAregisterGsmService");
    private static final QName QNAME_MARKE = new QName("http://ws.marke.register.dpma.de", "DPMAregisterMarkeService");
    private static final QName QNAME_PATENT = new QName("http://ws.patent.register.dpma.de", "DPMAregisterPatService");

    private final String serverUrlPart;
    private final String user;
    private final char[] pwd;

    public DPMAregisterClient(String serverUrlPart, String user, char[] pwd) {
        this.serverUrlPart = serverUrlPart;
        this.user = user;
        this.pwd = pwd;
    }

    public String getRegisterInfo(Schutzrecht schutzrecht, String akz) throws MalformedURLException {

        switch (schutzrecht) {
        case GESCHMACKSMUSTER:
            return getRegisterInfoGsm(akz);
        case MARKEN:
            return getRegisterInfoMarke(akz);
        case PATENTE:
            return getRegisterInfoPatent(akz);
        }

        throw new IllegalArgumentException(String.format("Unbekanntes Schutzrecht '%s'!", schutzrecht.name()));
    }

    private String getRegisterInfoGsm(String akz) throws MalformedURLException {

        String wsdlUrlStr = serverUrlPart + GSM_SERVICE_WSDL;
        DPMAregisterGsmService service = new DPMAregisterGsmService(new java.net.URL(wsdlUrlStr), QNAME_GSM);
        de.dpma.register.ws.gsm.DPMAregister endpoint = service.getPort(de.dpma.register.ws.gsm.DPMAregister.class);
        String endpointUrlStr = wsdlUrlStr.substring(0, wsdlUrlStr.length() - 5);
        ((BindingProvider)endpoint).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointUrlStr);        
        return endpoint.getRegisterInfo(user, new String(pwd), akz);
    }

    private String getRegisterInfoMarke(String akz) throws MalformedURLException {

        String wsdlUrlStr = serverUrlPart + MARKE_SERVICE_WSDL;
        DPMAregisterMarkeService service = new DPMAregisterMarkeService(new java.net.URL(wsdlUrlStr), QNAME_MARKE);
        de.dpma.register.ws.marke.DPMAregister endpoint = service.getPort(de.dpma.register.ws.marke.DPMAregister.class);
        String endpointUrlStr = wsdlUrlStr.substring(0, wsdlUrlStr.length() - 5);
        ((BindingProvider)endpoint).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointUrlStr);        
        return endpoint.getRegisterInfo(user, new String(pwd), akz);
    }

    private String getRegisterInfoPatent(String akz) throws MalformedURLException {

        String wsdlUrlStr = serverUrlPart + PATENT_SERVICE_WSDL;
        DPMAregisterPatService service = new DPMAregisterPatService(new java.net.URL(wsdlUrlStr), QNAME_PATENT);
        de.dpma.register.ws.patent.DPMAregister endpoint = service.getPort(de.dpma.register.ws.patent.DPMAregister.class);
        String endpointUrlStr = wsdlUrlStr.substring(0, wsdlUrlStr.length() - 5);
        ((BindingProvider)endpoint).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointUrlStr);        
        return endpoint.getRegisterInfo(user, new String(pwd), akz);
    }

    public String search(Schutzrecht schutzrecht, String query) throws MalformedURLException {

        switch (schutzrecht) {
        case GESCHMACKSMUSTER:
            return searchGsm(user, pwd, query);
        case MARKEN:
            return searchMarken(user, pwd, query);
        case PATENTE:
            return searchPatente(user, pwd, query);
        }

        throw new IllegalArgumentException(String.format("Unbekanntes Schutzrecht '%s'!", schutzrecht.name()));
    }

    private String searchGsm(String user, char[] pwd, String query) throws MalformedURLException {

        String wsdlUrlStr = serverUrlPart.toString() + GSM_SERVICE_WSDL;
        DPMAregisterGsmService service = new DPMAregisterGsmService(new java.net.URL(wsdlUrlStr), QNAME_GSM);
        de.dpma.register.ws.gsm.DPMAregister endpoint = service.getPort(de.dpma.register.ws.gsm.DPMAregister.class);
        String endpointUrlStr = wsdlUrlStr.substring(0, wsdlUrlStr.length() - 5);
        ((BindingProvider)endpoint).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointUrlStr);        
        return endpoint.search(user, new String(pwd), query);
    }

    private String searchMarken(String user, char[] pwd, String query) throws MalformedURLException {

        String wsdlUrlStr = serverUrlPart.toString() + MARKE_SERVICE_WSDL;
        DPMAregisterMarkeService service = new DPMAregisterMarkeService(new java.net.URL(wsdlUrlStr), QNAME_MARKE);
        de.dpma.register.ws.marke.DPMAregister endpoint = service.getPort(de.dpma.register.ws.marke.DPMAregister.class);
        String endpointUrlStr = wsdlUrlStr.substring(0, wsdlUrlStr.length() - 5);
        ((BindingProvider)endpoint).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointUrlStr);        
        return endpoint.search(user, new String(pwd), query);
    }

    private String searchPatente(String user, char[] pwd, String query) throws MalformedURLException {

        String wsdlUrlStr = serverUrlPart.toString() + PATENT_SERVICE_WSDL;
        DPMAregisterPatService service = new DPMAregisterPatService(new java.net.URL(wsdlUrlStr), QNAME_PATENT);
        de.dpma.register.ws.patent.DPMAregister endpoint = service.getPort(de.dpma.register.ws.patent.DPMAregister.class);
        String endpointUrlStr = wsdlUrlStr.substring(0, wsdlUrlStr.length() - 5);
        ((BindingProvider)endpoint).getRequestContext().put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, endpointUrlStr);        
        return endpoint.search(user, new String(pwd), query);
    }

    public static void main(String[] args) {
    	System.out.println("You can always stop by entering 'exit'.");
    	DPMAregisterClient service = null;

    	try {
    		String protocol = null, server = null, serverPort = null, user = null, password = null;
    		
    		for(String cmdArg : args) {
    			if(cmdArg.startsWith("-protocol=")) {
    				protocol = cmdArg.substring("-protocol=".length(), cmdArg.length());
    			}
    		}
    		if(isEmpty(protocol)) {
	    		System.out.print("Please enter the protocol (http/https, default https): ");
	    		protocol = getInput();
    		}
    		// Protokoll kann nur http oder https sein!
    		if(!"https".equalsIgnoreCase(protocol) && !"http".equalsIgnoreCase(protocol)) {
    			protocol = "https";
    			System.out.println("Protocol set to https...");
    		}
    		protocol = protocol.toLowerCase();
    		
    		for(String cmdArg : args) {
    			if(cmdArg.startsWith("-server=")) {
    				server = cmdArg.substring("-server=".length(), cmdArg.length());
    			}
    		}
    		if(isEmpty(server)) {
	    		System.out.print("Please enter the server address (default dpmaconnect.dpma.de): ");
	    		server = getInput();
	    		if(isEmpty(server)) server = "dpmaconnect.dpma.de";
    		}
    		
    		for(String cmdArg : args) {
    			if(cmdArg.startsWith("-serverPort=")) {
    				serverPort = cmdArg.substring("-serverPort=".length(), cmdArg.length());
    			}
    		}
    		if(isEmpty(serverPort)) {
    			String defaultPort = "https".equalsIgnoreCase(protocol) ? "443" : "80";
	    		System.out.print("Please enter the server port (default " + defaultPort + "): ");
	    		serverPort = getInput();
	    		if(isEmpty(serverPort)) serverPort = defaultPort;
    		}
    		

    		for(String cmdArg : args) {
    			if(cmdArg.startsWith("-user=")) {
    				user = cmdArg.substring("-user=".length(), cmdArg.length());
    			}
    		}
    		if(isEmpty(user)) {
	    		System.out.print("Please enter your web service user: ");
	    		user = getInput();
    		}
    		
    		for(String cmdArg : args) {
    			if(cmdArg.startsWith("-password=")) {
    				password = cmdArg.substring("-password=".length(), cmdArg.length());
    			}
    		}
    		if(isEmpty(password)) {
	    		System.out.print("Please enter your web service user password: ");
	    		password = getInput();
    		}
    		
    		if(isEmpty(user) || isEmpty(password)) {
    			System.out.println("Empty user and/or password. Exiting...");
                System.exit(1);
    		}
    		
    		
    		service = new DPMAregisterClient(new java.net.URL(protocol + "://" + server + ":" + serverPort).toString(), // new java.net.URL zum Test ob korrekte URL
    				user, password.toCharArray());
    	}
        catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    	
        while(true) {
	        try {
	        	System.out.print("Please enter the ip type (1 = designs, 2 = trade marks, 3 = patents): ");
	        	String ipType = getInput();
	        	Schutzrecht aSchutzrecht = null;
	        	if("1".equals(ipType)) aSchutzrecht = Schutzrecht.GESCHMACKSMUSTER;
	        	else if("2".equals(ipType)) aSchutzrecht = Schutzrecht.MARKEN;
	        	else if("3".equals(ipType)) aSchutzrecht = Schutzrecht.PATENTE;
	        	else {
	        		System.out.println("Invalid ip type...");
	        		continue;
	        	}

	        	System.out.print("Please enter the wanted operation (1 = search, 2 = getRegisterInfo): ");
	        	String operation = getInput();
	        	if("1".equals(operation)) {
		        	System.out.print("Please enter your query (e.g. INH=\"Coca Cola\"): ");
		        	String query = getInput();
		        	if(isEmpty(query)) System.out.println("Invalid query...");
		        	else System.out.println(service.search(aSchutzrecht, query));
	        	}
	        	else if("2".equals(operation)) {
	        		if("1".equals(ipType)) System.out.print("Please enter the wanted design identifier (gsnr): ");
	        		else if("2".equals(ipType)) System.out.print("Please enter the wanted application number (akz): ");
	        		else if("3".equals(ipType)) System.out.print("Please enter the wanted leading registered number (fakz): ");
		        	String akz = getInput();
		        	if(isEmpty(akz)) {
		        		if("1".equals(ipType)) System.out.println("Invalid design identifier (gsnr)...");
		        		if("2".equals(ipType)) System.out.println("Invalid application number (akz)...");
		        		else if("3".equals(ipType)) System.out.println("Invalid leading registered number (fakz)...");
		        	}
		        	else System.out.println(service.getRegisterInfo(aSchutzrecht, akz));
	        	}
	        	else {
	        		System.out.println("Invalid operation...");
	        	}
	        	
	        }
	        catch (Exception e) {
	            e.printStackTrace();
	        }
        }
    }

    private static String getInput() throws java.io.IOException {
    	try {
	        byte abyte[] = new byte[256];
	        int i = System.in.read(abyte);
	        String s = "";
	        if(i != -1) {
	        	s = (new String(abyte)).substring(0, i - 1).trim();
	        }
	        if("exit".equalsIgnoreCase(s)) {
	        	System.exit(0);
	        }
	        return s;
    	}
        catch(java.io.IOException ioexception) {
	        System.err.println((new StringBuilder()).append("An error has occurred while reading in command : ").append(ioexception).toString());
	        throw ioexception;
        }
    }
    
    private static boolean isEmpty(String s) {
    	return s == null || s.length() == 0;
    }
}
